# include "../main/utilities.h"

# include "../map/assignment.h"
# include "formula.h"
# include "formulaTest.h"



static std::string name = "formulaAndTest::";


FormulaAndTest::TestGetNumberOfElements::TestGetNumberOfElements() : Test() { }


FormulaAndTest::TestGetNumberOfElements::~TestGetNumberOfElements() { }


bool FormulaAndTest::TestGetNumberOfElements::runTestAfterFork () {
	return testGetNumberOfElements(); }


bool FormulaAndTest::TestGetNumberOfElements::testGetNumberOfElements() {
	std::string function = "testGetNumberOfElements:";
	int testNumber = 0;
	bool failed = false;
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	FormulaAnd *ref = new FormulaAnd(elements);

    testNumber++;
	if (ref->getNumberOfElements() != 0) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete ref;
	elements->push_back(NULL);
	ref = new FormulaAnd(elements);

    testNumber++;
	if (ref->getNumberOfElements() != 1) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete ref;
	elements->push_back(NULL);
	ref = new FormulaAnd(elements);

    testNumber++;
	if (ref->getNumberOfElements() != 2) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete elements;
	delete ref;
	return !failed; }


FormulaAndTest::TestGetElement::TestGetElement() : Test() { }


FormulaAndTest::TestGetElement::~TestGetElement() { }


bool FormulaAndTest::TestGetElement::runTestAfterFork () {
	return testGetElement(); }


bool FormulaAndTest::TestGetElement::testGetElement() {
	std::string function = "testGetElement:";
	int testNumber = 0;
	bool failed = false;
	FormulaFalse *f = new FormulaFalse();
	FormulaIdentif *i = new FormulaIdentif(0);
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	FormulaAnd *ref = new FormulaAnd(elements);

    testNumber++;
	if (ref->getElement(0) != NULL) {
		std::cout << name + function << testNumber << " return value not "
			"NULL\n";
		failed = true; }

	delete ref;

	elements->push_back(f);
	ref = new FormulaAnd(elements);

    testNumber++;
	if (ref->getElement(0) != f) {
		std::cout << name + function << testNumber << " wrong element "
			"returned\n";
		failed = true; }

	delete f;
	delete ref;

	elements->clear();
	elements->push_back(i);
	ref = new FormulaAnd(elements);

    testNumber++;
	if (ref->getElement(0) != i) {
		std::cout << name + function << testNumber << " wrong element "
			"returned\n";
		failed = true; }

	delete elements;
	delete i;
	delete ref;
	return !failed; }


FormulaAndTest::TestEquals::TestEquals() : Test() { }


FormulaAndTest::TestEquals::~TestEquals() { }


bool FormulaAndTest::TestEquals::runTestAfterFork () {
	return testEquals(); }


bool FormulaAndTest::TestEquals::testEquals() {
	std::string function = "testGetNumberOfElements:";
	int testNumber = 0;
	bool failed = false;
	FormulaFalse *f1 = new FormulaFalse();
	FormulaFalse *f2 = new FormulaFalse();
	
	std::vector<Formula*> *v = new std::vector<Formula*>();
	v->push_back(f1);
	v->push_back(f1);
	FormulaAnd *ref1 = new FormulaAnd(v);
	
	v->clear();
	v->push_back(f2);
	v->push_back(f2);
	FormulaAnd *ref2 = new FormulaAnd(v);

	FormulaIdentif *i = new FormulaIdentif(0);
	
	v->clear();
	v->push_back(i);
	v->push_back(i);
	FormulaAnd *ref3 = new FormulaAnd(v);

    testNumber++;
	if (! (ref1->equals(ref1))) {
		std::cout << name + function << testNumber << " ref1 SHOULD be equal "
			"to itself\n";
		failed = true; }

    testNumber++;
	if (! (ref1->equals(ref2))) {
		std::cout << name + function << testNumber << " ref1 SHOULD be equal "
			"to ref2\n";
		failed = true; }

    testNumber++;
	if (ref1->equals(ref3)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to ref3\n";
		failed = true; }

    testNumber++;
	if (ref1->equals(f1)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to a\n";
		failed = true; }

	delete v;
	delete f1;
	delete f2;
	delete ref1;
	delete ref2;
	delete i;
	delete ref3;
	return !failed; }


FormulaAndTest::TestGetValue::TestGetValue() : Test() { }


FormulaAndTest::TestGetValue::~TestGetValue() { }


bool FormulaAndTest::TestGetValue::runTestAfterFork () {
	return testGetValue(); }


bool FormulaAndTest::TestGetValue::testGetValue() {
	std::string function = "testGetNumberOfElements:";
	int testNumber = 0;
	bool failed = false;
	FormulaTrue *t = new FormulaTrue();
	FormulaIdentif *i1 = new FormulaIdentif(0);
	FormulaIdentif *i2 = new FormulaIdentif(1);
	FormulaIdentif *i3 = new FormulaIdentif(2);
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	elements->push_back(t);
	elements->push_back(i1);
	elements->push_back(i2);
	elements->push_back(i3);
	FormulaAnd *ref = new FormulaAnd(elements);

	Assignment *v = new Assignment(3);
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != true) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	delete elements;
	delete t;
	delete i1;
	delete i2;
	delete i3;
	delete ref;
	return !failed; }
