# include "../main/utilities.h"

# include "formula.h"
# include "formulaTest.h"



static std::string name = "formulaBoxTest::";


FormulaBoxTest::TestGetNumberOfElements::TestGetNumberOfElements() : Test() { }


FormulaBoxTest::TestGetNumberOfElements::~TestGetNumberOfElements() { }


bool FormulaBoxTest::TestGetNumberOfElements::runTestAfterFork () {
	return testGetNumberOfElements(); }


bool FormulaBoxTest::TestGetNumberOfElements::testGetNumberOfElements() {
	std::string function = "testGetNumberOfElements:";
	int testNumber = 0;
	bool failed = false;
	Formula *element = new FormulaTrue();
	FormulaBox *ref = new FormulaBox(NULL);

    testNumber++;
	if (ref->getNumberOfElements() != 1) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete ref;
	ref = new FormulaBox(element);

    testNumber++;
	if (ref->getNumberOfElements() != 1) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete ref;
	delete element;
	element = new FormulaIdentif(0);
	ref = new FormulaBox(element);

    testNumber++;
	if (ref->getNumberOfElements() != 1) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete element;
	delete ref;
	return !failed; }


FormulaBoxTest::TestGetElement::TestGetElement() : Test() { }


FormulaBoxTest::TestGetElement::~TestGetElement() { }


bool FormulaBoxTest::TestGetElement::runTestAfterFork () {
	return testGetElement(); }


bool FormulaBoxTest::TestGetElement::testGetElement() {
	std::string function = "testGetElement:";
	int testNumber = 0;
	bool failed = false;
	FormulaFalse *f = new FormulaFalse();
	FormulaIdentif *i = new FormulaIdentif(0);
	FormulaBox *ref = new FormulaBox(NULL);

    testNumber++;
	if (ref->getElement(0) != NULL) {
		std::cout << name + function << testNumber << " return value not "
			"NULL\n";
		failed = true; }

	delete ref;

	ref = new FormulaBox(f);

    testNumber++;
	if (ref->getElement(0) != f) {
		std::cout << name + function << testNumber << " wrong element "
			"returned\n";
		failed = true; }

	delete f;
	delete ref;
	ref = new FormulaBox(i);

    testNumber++;
	if (ref->getElement(0) != i) {
		std::cout << name + function << testNumber << " wrong element "
			"returned\n";
		failed = true; }

	delete i;
	delete ref;
	return !failed; }


FormulaBoxTest::TestEquals::TestEquals() : Test() { }


FormulaBoxTest::TestEquals::~TestEquals() { }


bool FormulaBoxTest::TestEquals::runTestAfterFork () {
	return testEquals(); }


bool FormulaBoxTest::TestEquals::testEquals() {
	std::string function = "testEquals:";
	int testNumber = 0;
	bool failed = false;
	FormulaBox *ref1 = new FormulaBox(NULL);
	FormulaBox *ref2 = new FormulaBox(NULL);
	FormulaTrue *ft = new FormulaTrue();
	FormulaBox *ref3 = new FormulaBox(ft);
	FormulaTrue *ref4 = new FormulaTrue();

	std::vector<Formula*> *ar = new std::vector<Formula*>();
	ar->push_back(ref1);
	ar->push_back(ref2);
	FormulaAnd *ref5 = new FormulaAnd(ar);

    testNumber++;
	if (! (ref1->equals(ref1))) {
		std::cout << name + function << testNumber << " ref1 SHOULD be equal "
			"to itself\n";
		failed = 1; }

    testNumber++;
	if (! (ref1->equals(ref2))) {
		std::cout << name + function << testNumber << " ref1 SHOULD be equal "
			"to ref2\n";
		failed = 1; }

    testNumber++;
	if (ref1->equals(ref3)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to ref3\n";
		failed = 1; }

    testNumber++;
	if (ref1->equals(ref4)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to ref4\n";
		failed = 1; }

    testNumber++;
	if (ref1->equals(ref5)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to ref5\n";
		failed = 1; }

	delete ar;
	ref3->erase();
	ref4->erase();
	ref5->erase();
	return !failed; }


FormulaBoxTest::TestGetValue::TestGetValue() : Test() { }


FormulaBoxTest::TestGetValue::~TestGetValue() { }


bool FormulaBoxTest::TestGetValue::runTestAfterFork () {
	return testGetValue(); }


bool FormulaBoxTest::TestGetValue::testGetValue() {
	std::string function = "testGetNumberOfElements:";
	int testNumber = 0;
	bool failed = false;
	FormulaTrue *t = new FormulaTrue();
	FormulaIdentif *i1 = new FormulaIdentif(0);
	FormulaIdentif *i2 = new FormulaIdentif(1);
	FormulaIdentif *i3 = new FormulaIdentif(2);
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	elements->push_back(t);
	elements->push_back(i1);
	elements->push_back(i2);
	elements->push_back(i3);
	FormulaAnd *fa = new FormulaAnd(elements);
	FormulaBox *ref = new FormulaBox(fa);

	Assignment *v = new Assignment(3);
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != fa->getValue(v)) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	delete elements;
	delete t;
	delete i1;
	delete i2;
	delete i3;
	delete fa;
	delete ref;
	return !failed; }
