# include "../main/utilities.h"

# include "formulaStamp.h"
# include "formulaTable.h"


/*
 * @param stamp: the ID of the formula this FormulaStamp points to.
 *              This must be a valid ID from the global formula table
 *              (the variable 'table').
 */
FormulaStamp::FormulaStamp(int stamp): Formula() {
	this->stamp = stamp; }


FormulaStamp::~FormulaStamp() { }


/*
 * Returns the number of elements of the formula pointed by this instance
 * of FormulaStamp.
 */
int FormulaStamp::getNumberOfElements() {
	if (table->getFormula(stamp) != NULL) {
		return table->getFormula(stamp)->getNumberOfElements(); }
	return 0; }


/*
 * @param index: the index of the element to be retrieved.
 * Returns the element at index of the formula pointed by this instance
 * of FormulaStamp, if possible. Returns NULL otherwise.
 */
Formula* FormulaStamp::getElement (int index) {
	if (table->getFormula(stamp) != NULL) {
		return table->getFormula(stamp)->getElement(index); }
	return NULL; }


/*
 * Returns the IDValue of the formula pointed by this instance
 * of FormulaStamp, if possible. Returns -1 otherwise.
 */
int FormulaStamp::getIDValue () {
	if (table->getFormula(stamp) != NULL) {
		return table->getFormula(stamp)->getIDValue(); }
	return -1; }


/*
 * @param f: the formula to be compared.
 * Returns true if f is a FormulaStamp and points to the same formula
 * that this instance points. Returns false otherwise.
 */
bool FormulaStamp::equals (Formula *f) {
	FormulaStamp *fs = dynamic_cast<FormulaStamp *>(f);

	if (fs == NULL) return false;
	if (fs->stamp != stamp) return false;

	return true; }


/*
 * @param v: the assignment to be applied.
 * Returns the value of the formula pointed by this instance
 * of FormulaStamp, when v is applied. Returns false if no formula is
 * pointed
 */
bool FormulaStamp::getValue (Assignment *v) {
	if (table->getFormula(stamp) != NULL) {
		return table->getFormula(stamp)->getValue(v); }
	return false; }


/*
 * prints the formula pointed by this instance of FormulaStamp in the
 * standard output.
 */
void FormulaStamp::print() {
	if (table->getFormula(stamp) != NULL) table->getFormula(stamp)->print(); }


/*
 * Returns the ID of the formula pointed by this instance of
 * FormulaStamp. This value is associated with the global formula table
 */
int FormulaStamp::getStamp() {
	return stamp; }
