# include "../main/utilities.h"

# include "formulaStamp.h"
# include "formulaTable.h"


FormulaTable *table;


FormulaTable::FormulaTable () {
	tableElements = new std::vector<Formula*>();
	formulasIdentif = new std::vector<unsigned>();
	formulasBox = new std::vector<unsigned>();
	formulasAnd = new std::vector<unsigned>();
	formulasXor = new std::vector<unsigned>();
	sequents = new std::vector<unsigned>();
	tableElements->push_back(new FormulaTrue());
	tableElements->push_back(new FormulaFalse()); }


FormulaTable::~FormulaTable () {
	for (unsigned i = 0; i < tableElements->size(); i++) delete tableElements->at(i);
	delete tableElements;
	delete formulasIdentif;
	delete formulasBox;
	delete formulasAnd;
	delete formulasXor;
	delete sequents; }


/*
 * Fetches a FormulaStamp that points to a FormulaTrue in the global
 * formula table.
 */
FormulaStamp* FormulaTable::fetchFormulaTrue() {
	return new FormulaStamp(0); }


/*
 * Fetches a FormulaStamp that points to a FormulaTrue in the global
 * formula table.
 */
FormulaStamp* FormulaTable::fetchFormulaFalse() {
	return new FormulaStamp(1); }


/*
 * @param IDValue: the IDValue of the requested formula
 * Fetches a FormulaStamp with the desired IDValue.
 */
FormulaStamp* FormulaTable::fetchFormulaIdentif(int IDValue) {
	FormulaIdentif *f = new FormulaIdentif(IDValue);

	unsigned i, j;
	for (i = 0; i < formulasIdentif->size(); i++) {
		j = formulasIdentif->at(i);
		if (tableElements->at(j)->equals(f)) {
			delete f;
			return new FormulaStamp(j); }}

	tableElements->push_back(f);
	formulasIdentif->push_back(tableElements->size() - 1);
	return new FormulaStamp (tableElements->size() - 1); }


/*
 * @param element: the element of the requested Box formula.
 *					note that if it isn't a FormulaStamp, it
 *					will be discarded.
 * Fetches a FormulaStamp that points to a FormulaBox with the desired
 * element, in the global formula table.
 */
FormulaStamp* FormulaTable::fetchFormulaBox(Formula *element) {
	FormulaBox *f = new FormulaBox(element);

	unsigned i, j;
	for (i = 0; i < formulasBox->size(); i++) {
		j = formulasBox->at(i);
		if (tableElements->at(j)->equals(f)) {
			delete f;
			return new FormulaStamp(j); }}

	tableElements->push_back(f);
	formulasBox->push_back(tableElements->size() - 1);
	return new FormulaStamp (tableElements->size() - 1); }


/*
 * @param elements: the elements of the requested And formula.
 *					note that any element that isn't a FormulaStamp
 *					will be discarded.
 * Fetches a FormulaStamp that points to a FormulaAnd with the desired
 * elements, in the global formula table.
 */
FormulaStamp* FormulaTable::fetchFormulaAnd(std::vector<Formula*> *elements) {
	FormulaAnd *f = new FormulaAnd(orderElements(elements));

	unsigned i, j;
	for (i = 0; i < formulasAnd->size(); i++) {
		j = formulasAnd->at(i);
		if (tableElements->at(j)->equals(f)) {
			delete f;
			return new FormulaStamp(j); }}

	tableElements->push_back(f);
	formulasAnd->push_back(tableElements->size() - 1);
	return new FormulaStamp (tableElements->size() - 1); }


/*
 * @param elements: the elements of the requested Xor formula.
 *					note that any element that isn't a FormulaStamp
 *					will be discarded.
 * Fetches a FormulaStamp that points to a FormulaXor with the desired
 * elements, in the global formula table.
 */
FormulaStamp* FormulaTable::fetchFormulaXor(std::vector<Formula*> *elements) {
	FormulaXor *f = new FormulaXor(orderElements(elements));

	unsigned i, j;
	for (i = 0; i < formulasXor->size(); i++) {
		j = formulasXor->at(i);
		if (tableElements->at(j)->equals(f)) {
			delete f;
			return new FormulaStamp(j); }}

	tableElements->push_back(f);
	formulasXor->push_back(tableElements->size() - 1);
	return new FormulaStamp (tableElements->size() - 1); }


/*
 * @param premises: the premises of the requested Sequent.
 *					note that any premise that isn't a FormulaStamp
 *					will be discarded.
 * @param conclusion: the conclusion of the requested Sequent.
 *					note that if the conclusion isn't a FormulaStamp, it
 *					will be discarded.
 * Fetches a FormulaStamp that points to a Sequent with the desired
 * parameters, in the global formula table.
 */
FormulaStamp* FormulaTable::fetchSequent(std::vector<Formula*> *premises,
		Formula *conclusion) {
	Sequent *f;
	FormulaStamp *coreConclusion = dynamic_cast<FormulaStamp *>(conclusion);
	if (coreConclusion != NULL) {
		f = new Sequent(orderElements(premises),
			getFormula(coreConclusion->getStamp())); }
	else f = new Sequent(orderElements(premises), NULL);

	unsigned i, j;
	for (i = 0; i < sequents->size(); i++) {
		j = sequents->at(i);
		if (tableElements->at(j)->equals(f)) {
			delete f;
			tableElements->at(j)->print();
			return new FormulaStamp(j); }}

	tableElements->push_back(f);
	sequents->push_back(tableElements->size() - 1);
	return new FormulaStamp (tableElements->size() - 1); }


/*
 * @param IDValue: the IDValue of the requested formula
 * Fetches a FormulaStamp with the desired IDValue.
 */
Formula* FormulaTable::getFormula(unsigned stamp) {
	if (stamp >= tableElements->size()) return NULL;
	return tableElements->at(stamp); }


/*
 * @param elements: the elements needed to be sorted.
 *					note that any element that isn't a FormulaStamp
 *					will be discarded.
 * Returns a vector of Formulas ordered by their stamp values in the
 * global formula table. Note that the elements of such vector are the
 * actual formulas, not the original FormulaStamps pointing to them.
 */
std::vector<Formula*> *FormulaTable::orderElements(
		std::vector<Formula*> *elements) {
	unsigned i, j, min, last;
	int *stamps;
	
	last = elements->size();
	if (last == 0) return new std::vector<Formula*>();
	stamps = (int *) malloc (last*sizeof(int));

	FormulaStamp *item;
	for (i = 0; i < elements->size(); i++) {
		item = dynamic_cast<FormulaStamp *>(elements->at(i));
		if (item != NULL) stamps[i] = item->getStamp();
		else stamps[i] = -1; }

	std::vector<Formula*> *orderedElements = new std::vector<Formula*>();
	orderedElements->reserve(last);
	for (i = min = 0; i < last; i++, min = 0) {
		for (j = 1; j < last; j++) {
			if (stamps[min] == -1 ||
					(stamps[j] != -1 && stamps[min] > stamps[j])) {
				min = j; }}
		if (stamps[min] != -1) {
			orderedElements->push_back(getFormula(stamps[min])); }
		stamps[min] = -1; }

	free(stamps);
	return orderedElements; }
