# ifndef FORMULA_TABLE_H
# define FORMULA_TABLE_H

# include <vector>
# include "formulaStamp.h"

/**
 *
 * This class wraps a table for formulas
 *
 */
class FormulaTable {
	public:
		FormulaTable ();
		~FormulaTable ();

		/*
		 * Fetches a FormulaStamp that points to a FormulaTrue in the global
		 * formula table.
		 */
		FormulaStamp *fetchFormulaTrue();

		/*
		 * Fetches a FormulaStamp that points to a FormulaTrue in the global
		 * formula table.
		 */
		FormulaStamp *fetchFormulaFalse();

		/*
		 * @param IDValue: the IDValue of the requested formula
		 * Fetches a FormulaStamp with the desired IDValue.
		 */
		FormulaStamp *fetchFormulaIdentif(int IDValue);

		/*
		 * @param element: the element of the requested Box formula.
		 *					note that if it isn't a FormulaStamp, it
		 *					will be discarded.
		 * Fetches a FormulaStamp that points to a FormulaBox with the desired
		 * element, in the global formula table.
		 */
		FormulaStamp *fetchFormulaBox(Formula *element);

		/*
		 * @param elements: the elements of the requested And formula.
		 *					note that any element that isn't a FormulaStamp
		 *					will be discarded.
		 * Fetches a FormulaStamp that points to a FormulaAnd with the desired
		 * elements, in the global formula table.
		 */
		FormulaStamp *fetchFormulaAnd(std::vector<Formula*> *elements);

		/*
		 * @param elements: the elements of the requested Xor formula.
		 *					note that any element that isn't a FormulaStamp
		 *					will be discarded.
		 * Fetches a FormulaStamp that points to a FormulaXor with the desired
		 * elements, in the global formula table.
		 */
		FormulaStamp *fetchFormulaXor(std::vector<Formula*> *elements);

		/*
		 * @param premises: the premises of the requested Sequent.
		 *					note that any premise that isn't a FormulaStamp
		 *					will be discarded.
		 * @param conclusion: the conclusion of the requested Sequent.
		 *					note that if the conclusion isn't a FormulaStamp, it
		 *					will be discarded.
		 * Fetches a FormulaStamp that points to a Sequent with the desired
		 * parameters, in the global formula table.
		 */
		FormulaStamp *fetchSequent(std::vector<Formula*> *premises,
				Formula *conclusion);

		/*
		 * @param IDValue: the IDValue of the requested formula
		 * Fetches a FormulaStamp with the desired IDValue.
		 */
        Formula *getFormula(unsigned stamp);

	private:

		/*
		 * @param elements: the elements needed to be sorted.
		 *					note that any element that isn't a FormulaStamp
		 *					will be discarded.
		 * Returns a vector of Formulas ordered by their stamp values in the
		 * global formula table. Note that the elements of such vector are the
		 * actual formulas, not the original FormulaStamps pointing to them.
		 */
        std::vector<Formula*> *orderElements(std::vector<Formula*> *elements);

		std::vector<Formula*> *tableElements;
		std::vector<unsigned> *formulasIdentif;
		std::vector<unsigned> *formulasBox;
		std::vector<unsigned> *formulasAnd;
		std::vector<unsigned> *formulasXor;
		std::vector<unsigned> *sequents; };


// This is the global formula table
extern FormulaTable *table;



# endif // FROMULA_TABLE_H
