# include "../main/utilities.h"

# include "formula.h"
# include "formulaTest.h"



static std::string name = "formulaXorTest::";


FormulaXorTest::TestGetNumberOfElements::TestGetNumberOfElements() : Test() { }


FormulaXorTest::TestGetNumberOfElements::~TestGetNumberOfElements() { }


bool FormulaXorTest::TestGetNumberOfElements::runTestAfterFork () {
	return testGetNumberOfElements(); }


bool FormulaXorTest::TestGetNumberOfElements::testGetNumberOfElements() {
	std::string function = "testGetNumberOfElements:";
	int testNumber = 0;
	bool failed = false;
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	FormulaXor *ref = new FormulaXor(elements);

    testNumber++;
	if (ref->getNumberOfElements() != 0) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete ref;
	elements->push_back(new FormulaTrue());
	ref = new FormulaXor(elements);

    testNumber++;
	if (ref->getNumberOfElements() != 1) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	delete ref;
	elements->push_back(new FormulaTrue());
	ref = new FormulaXor(elements);

    testNumber++;
	if (ref->getNumberOfElements() != 2) {
		std::cout << name + function << testNumber << " wrong return value\n";
		failed = true; }

	ref->erase();
	delete elements;
	return !failed; }


FormulaXorTest::TestGetElement::TestGetElement() : Test() { }


FormulaXorTest::TestGetElement::~TestGetElement() { }


bool FormulaXorTest::TestGetElement::runTestAfterFork () {
	return testGetElement(); }


bool FormulaXorTest::TestGetElement::testGetElement() {
	std::string function = "testGetElement:";
	int testNumber = 0;
	bool failed = false;
	FormulaFalse *f = new FormulaFalse();
	FormulaIdentif *i = new FormulaIdentif(0);
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	FormulaXor *ref = new FormulaXor(elements);

    testNumber++;
	if (ref->getElement(0) != NULL) {
		std::cout << name + function << testNumber << " return value not "
			"NULL\n";
		failed = true; }

	delete ref;

	elements->push_back(f);
	ref = new FormulaXor(elements);

    testNumber++;
	if (ref->getElement(0) != f) {
		std::cout << name + function << testNumber << " wrong element "
			"returned\n";
		failed = true; }

	ref->erase();

	elements->clear();
	elements->push_back(i);
	ref = new FormulaXor(elements);

    testNumber++;
	if (ref->getElement(0) != i) {
		std::cout << name + function << testNumber << " wrong element "
			"returned\n";
		failed = true; }

	delete elements;
	ref->erase();
	return !failed; }


FormulaXorTest::TestEquals::TestEquals() : Test() { }


FormulaXorTest::TestEquals::~TestEquals() { }


bool FormulaXorTest::TestEquals::runTestAfterFork () {
	return testEquals(); }


bool FormulaXorTest::TestEquals::testEquals() {
	std::string function = "testEquals:";
	int testNumber = 0;
	bool failed = false;
	FormulaFalse *f = new FormulaFalse();

	std::vector<Formula*> *v1 = new std::vector<Formula*>();
	v1->push_back(new FormulaFalse());
	v1->push_back(new FormulaFalse());
	FormulaXor *ref1 = new FormulaXor(v1);

	std::vector<Formula*> *v2 = new std::vector<Formula*>();
	v2->push_back(new FormulaFalse());
	v2->push_back(new FormulaFalse());
	FormulaXor *ref2 = new FormulaXor(v2);

	std::vector<Formula*> *v3 = new std::vector<Formula*>();
	v3->push_back(new FormulaIdentif(0));
	v3->push_back(new FormulaIdentif(0));
	FormulaXor *ref3 = new FormulaXor(v3);

    testNumber++;
	if (! (ref1->equals(ref1))) {
		std::cout << name + function << testNumber << " ref1 SHOULD be equal "
			"to itself\n";
		failed = true; }

    testNumber++;
	if (! (ref1->equals(ref2))) {
		std::cout << name + function << testNumber << " ref1 SHOULD be equal "
			"to ref2\n";
		failed = true; }

    testNumber++;
	if (ref1->equals(ref3)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to ref3\n";
		failed = true; }

    testNumber++;
	if (ref1->equals(f)) {
		std::cout << name + function << testNumber << " ref1 should NOT be "
			"equal to f\n";
		failed = true; }

	delete v1;
	delete v2;
	delete v3;
	delete f;
	ref1->erase();
	ref2->erase();
	ref3->erase();
	return !failed; }


FormulaXorTest::TestGetValue::TestGetValue() : Test() { }


FormulaXorTest::TestGetValue::~TestGetValue() { }


bool FormulaXorTest::TestGetValue::runTestAfterFork () {
	return testGetValue(); }


bool FormulaXorTest::TestGetValue::testGetValue() {
	std::string function = "testGetValue:";
	int testNumber = 0;
	bool failed = false;
	std::vector<Formula*> *elements = new std::vector<Formula*>();
	elements->push_back(new FormulaTrue());
	elements->push_back(new FormulaIdentif(0));
	elements->push_back(new FormulaIdentif(1));
	elements->push_back(new FormulaIdentif(2));
	FormulaXor *ref = new FormulaXor(elements);

	Assignment *v = new Assignment(3);
    testNumber++;
	if (ref->getValue(v) != true) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		return 1; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != true) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != true) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != true) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	v->nextValueSet();
    testNumber++;
	if (ref->getValue(v) != false) {
		std::cout << name + function << testNumber << " wrong value "
			"returned\n";
		failed = true; }

	delete elements;
	ref->erase();
	return !failed; }
