# include "utilities.h"


static bool debugActive = false;
static int silence = 0;
static int watchValue = 0;
static std::ofstream outFile;
static int formulaCount = 0, maxFormulas = 0;
static bool plainMethod = true, tableMethod = false;


/*
 * Sets the method of calculus to be used in the program.
 * methodIndex = 0: plain method
 * methodIndex = 1: table method
 */
void setMethod(int methodIndex) {
	switch (methodIndex) {
		case 0:
			plainMethod = true;
			tableMethod = false;
			break;
		case 1:
			plainMethod = false;
			tableMethod = true;
			break; }}


/*
 * Returns true is the plain method is to be used.
 * Returns false otherwise.
 */
bool isPlainMethodInUse() {
	return plainMethod; }


/*
 * Returns true is the table method is to be used.
 * Returns false otherwise.
 */
bool isTableMethodInUse() {
	return tableMethod; }


/*
 * Sets the level of silence for operations of the program.
 */
void setSilence(int value) {
	silence = value; }


/*
 * Returns the level of silence for operations of the program.
 */
int getSilence() {
	return silence; }


/*
 * Sets an output file for profile analysis.
 */
void setOutFile(const char* name) {
	if (outFile.is_open()) outFile.close();
	outFile.open (name); }


/*
 * Returns an output file for profile analysis.
 */
std::ofstream* getOutFile() {
	return &outFile; }


/*
 * Increments the count of total formulas in use.
 */
void formulasIncrement() {
	formulaCount++;
	if (formulaCount > maxFormulas) maxFormulas = formulaCount; }


/*
 * Decrements the count of total formulas in use.
 */
void formulasDecrement() {
	formulaCount--; }


/*
 * Returns the number of total formulas in use.
 */
int getFormulaCount(){
	return formulaCount; }


/*
 * Returns the highest number of formulas used in any given time.
 */
int getMaxFormulas() {
	return maxFormulas; }


/*
 * Resets the number of formulas used in any given time.
 */
void resetMaxFormulas() {
	maxFormulas = 0; }


/*
 * Pauses the program (note only works in Windows operational system)
 */
void utilitiesPause() {
	system("PAUSE"); }


/*
 * Swithces debug mode on/off.
 */
void switchDebugging() {
	debugActive = !debugActive; }


/*
 * @param value: true if debug mode should be activated. false, to deactivate it
 * Sets debug mode to @value.
 */
void setDebugging(bool value) {
	debugActive = value; }


/*
 * Returns true if debug mode is activated. Returns false otherwise.
 */
bool isDebugging() {
	return debugActive; }
