# ifndef UTILITIES_H
# define UTILITIES_H

# include <cstdlib>
# include <iostream>
# include <fstream>


/*
 * Sets the method of calculus to be used in the program.
 * methodIndex = 0: plain method
 * methodIndex = 1: table method
 */
void setMethod(int methodIndex);

/*
 * Returns true is the plain method is to be used.
 * Returns false otherwise.
 */
bool isPlainMethodInUse();

/*
 * Returns true is the table method is to be used.
 * Returns false otherwise.
 */
bool isTableMethodInUse();

/*
 * Sets the level of silence for operations of the program.
 */
void setSilence(int value);
/*
 * Returns the level of silence for operations of the program.
 */
int getSilence();

/*
 * Sets an output file for profile analysis.
 */
void setOutFile(const char* name);
/*
 * Returns an output file for profile analysis.
 */
std::ofstream* getOutFile();

/*
 * Increments the count of total formulas in use.
 */
void formulasIncrement();
/*
 * Decrements the count of total formulas in use.
 */
void formulasDecrement();
/*
 * Returns the number of total formulas in use.
 */
int getFormulaCount();
/*
 * Returns the highest number of formulas used in any given time.
 */
int getMaxFormulas();
/*
 * Resets the number of formulas used in any given time.
 */
void resetMaxFormulas();

/*
 * Pauses the program (note only works in Windows operational system)
 */
void utilitiesPause();

/*
 * Swithces debug mode on/off.
 */
void switchDebugging();
/*
 * @param value: true if debug mode should be activated. false, to deactivate it
 * Sets debug mode to @value.
 */
void setDebugging(bool value);
/*
 * Returns true if debug mode is activated. Returns false otherwise.
 */
bool isDebugging();

# endif // UTILITIES_H
