# include "../main/utilities.h"

# include "parser.h"
# include "stack.h"
# include "../formula/formulaStamp.h"



Parser::Parser(std::string &s) {
	std::string *str_ptr = clearBadChars(s);
	this->s = *(str_ptr) + "#";
	delete str_ptr;
	mapper = new Map(); }


Parser::~Parser() {
	delete mapper; }


Formula* Parser::parse() {
	read = lastTokenInit = -1;
	formulaStack = new Stack();
	
	if (getEntireFormula() != true) {
		while (formulaStack->getSize() > 0) {
			Formula *f = formulaStack->pop();
			f->erase(); }
		delete formulaStack;
		return NULL; }

	Formula *f = formulaStack->pop();

	delete formulaStack;
	return f; }


Map* Parser::getMap() {
	return mapper; }


bool Parser::getEntireFormula() {
	while (true) {
		if (getNextToken() == TOKEN_TERMINAL) return true;
		rollBackLastToken();
		if (!getFormula()) return false; }}


bool Parser::getFormula() {
	bool result = true;
	enum token t = getNextToken();
		
	switch (t) {
		case TOKEN_OPEN:
			if (!getFormula()) result = false;
			if (!getComposedFormula()) result = false;
			break;
		case TOKEN_BOX:
			if (!getBoxFormula()) result = false;
			break;
		case TOKEN_NOT:
			if (!getNotFormula()) result = false;
			break;
		case TOKEN_AND:
		case TOKEN_OR:
			rollBackLastToken();
		case TOKEN_IDENTIF:
			if (!getIdentifierFormula()) result = false;
			break;
		case TOKEN_COMMA:
			if (formulaStack->getSize() != 1) return false;
			if (!getSequent(TOKEN_COMMA)) result = false;
			break;
		case TOKEN_DASH:
			if (!getSequent(TOKEN_DASH)) result = false;
			break;
		case TOKEN_CLOSE:
		case TOKEN_TERMINAL:
		default:
			result = false; }

	return result; }


bool Parser::getSequent(TokenType firstToken) {
	if (firstToken == TOKEN_COMMA) {
		while (true) {
			if (!getFormula()) return false;
			
			TokenType t = getNextToken();
			if (t == TOKEN_COMMA) continue;
			if (t == TOKEN_DASH) {
				firstToken = TOKEN_DASH;
				break; }}}
	
	if (firstToken == TOKEN_DASH) {

		if (!getFormula()) return false;

		Formula *conclusion = formulaStack->pop();

		std::vector<Formula*> *premises = new std::vector<Formula*>();
		while (formulaStack->getSize() > 0) {
			premises->push_back(formulaStack->pop()); }

		Formula *s = createSequent(premises, conclusion);
		formulaStack->push(s);
		return true; }
	
	return false; }


bool Parser::getComposedFormula() {
	bool result = true;
	TokenType t = getNextToken();
		
	switch (t) {
		case TOKEN_AND:
			if (!getAndFormula()) result = false;
			break;
		case TOKEN_OR:
			if (!getOrFormula()) result = false;
			break;
		case TOKEN_IMPLY:
			if (!getImplyFormula()) result = false;
			break;
		case TOKEN_NOT:
		case TOKEN_IDENTIF:
		case TOKEN_OPEN:
		case TOKEN_CLOSE:
		case TOKEN_COMMA:
		case TOKEN_DASH:
		case TOKEN_TERMINAL:
		default:
			result = false; }

	return result; }


bool Parser::getImplyFormula () {
	if (getNextToken() != TOKEN_OPEN) return false;
	if (getFormula() != true) return false;

	std::vector<Formula*> *elements = new std::vector<Formula*>();

	elements->push_back(formulaStack->pop());
	elements->push_back(createFormulaTrue());
	Formula *formula = createFormulaXor(elements);
	elements->clear();

	elements->push_back(formulaStack->pop());
	elements->push_back(formula);
	formula = createFormulaAnd(elements);
	elements->clear();

	elements->push_back(formula);
	elements->push_back(createFormulaTrue());
	formulaStack->push(createFormulaXor(elements));
	delete elements;
	
	TokenType t = getNextToken();
	if (t != TOKEN_CLOSE && t != TOKEN_COMMA && t != TOKEN_DASH &&
			t != TOKEN_TERMINAL) {
		return false; }
	if (t != TOKEN_CLOSE) rollBackLastToken();

	return true; }


bool Parser::getAndFormula () {
	TokenType t;
	bool result = true;

	std::vector<Formula*> *elements = new std::vector<Formula*>();
	elements->push_back(formulaStack->pop());

	while (result) {
		t = getNextToken();
		if (t != TOKEN_OPEN) result = false;
		if (getFormula() != true) result = false;

		elements->push_back(formulaStack->pop());

		t = getNextToken();
		if (t == TOKEN_CLOSE || t == TOKEN_COMMA ||	t == TOKEN_DASH ||
				t == TOKEN_TERMINAL) {
			if (t != TOKEN_CLOSE) rollBackLastToken();
			formulaStack->push(createFormulaAnd(elements));
			delete elements;
			return result; }

		if (t != TOKEN_AND) result = false; }

	std::vector<Formula*>::iterator it;
	for (it = elements->begin(); it != elements->end(); it++) {
		(*(it))->erase(); }
	delete elements;
	
	return result; }


bool Parser::getOrFormula () {
	enum token t;
	bool result = true;

	std::vector<Formula*> *elements = new std::vector<Formula*>();
	std::vector<Formula*> *v = new std::vector<Formula*>();
	v->push_back(formulaStack->pop());
	v->push_back(createFormulaTrue());
	elements->push_back(createFormulaXor(v));
	v->clear();

	while (result) {
		t = getNextToken();
		if (t != TOKEN_OPEN) result = false;
		if (getFormula() != true) result = false;

		v->push_back(formulaStack->pop());
		v->push_back(createFormulaTrue());
		elements->push_back(createFormulaXor(v));
		v->clear();

		t = getNextToken();
		if (t == TOKEN_CLOSE || t == TOKEN_COMMA || t == TOKEN_DASH ||
				t == TOKEN_TERMINAL) {
			if (t != TOKEN_CLOSE) rollBackLastToken();
			v->push_back(createFormulaAnd(elements));
			v->push_back(createFormulaTrue());

			formulaStack->push(createFormulaXor(v));
			delete v;
			delete elements;
			return true; }

		if (t != TOKEN_OR) result = false; }

	std::vector<Formula*>::iterator it;
	for (it = elements->begin(); it != elements->end(); it++) {
		(*(it))->erase(); }
	delete elements;
	
	return result; }


bool Parser::getBoxFormula () {
	if (getNextToken() != TOKEN_OPEN) return false;
	if (getFormula() != true) return false;

	formulaStack->push(createFormulaBox(formulaStack->pop()));

	TokenType t = getNextToken();
	if (t != TOKEN_CLOSE && t != TOKEN_COMMA && t != TOKEN_DASH &&
			t != TOKEN_TERMINAL) {
		return false; }
	if (t != TOKEN_CLOSE) rollBackLastToken();
	return true; }


bool Parser::getNotFormula () {
	if (getNextToken() != TOKEN_OPEN) return false;
	if (getFormula() != true) return false;

	std::vector<Formula*> *elements = new std::vector<Formula*>();

	elements->push_back(formulaStack->pop());
	elements->push_back(createFormulaTrue());
	formulaStack->push(createFormulaXor(elements));
	delete elements;

	TokenType t = getNextToken();
	if (t != TOKEN_CLOSE && t != TOKEN_COMMA && t != TOKEN_DASH &&
			t != TOKEN_TERMINAL) {
		return false; }
	if (t != TOKEN_CLOSE) rollBackLastToken();
	return true; }


bool Parser::getIdentifierFormula() {
	std::string identif = "";

	while (nextCharIsLetter() || nextCharIsDigit()) identif += nextChar();
    formulaStack->push(createFormulaIdentif(mapper->getValue(&identif)));
	
	TokenType t = getNextToken();
	if (t != TOKEN_CLOSE && t != TOKEN_COMMA && t != TOKEN_DASH &&
			t != TOKEN_TERMINAL) {
		return false; }

	if (t != TOKEN_CLOSE) rollBackLastToken();
	return true; }


bool Parser::isBlank(char c) {
	if (c == ' ' || c == '\t' || c == '\n') return true;
	return false; }


void Parser::readBlank() {
	while (read+1 < (signed) s.size()) {
		char next = s[read+1];
		if (!isBlank(next)) return;
		read++; }}


bool Parser::nextCharIsDigit() {
	if (read+1 >= (signed) s.size()) return false;

	switch (s[read+1]) {
		case '1': case '2': case '3': case '4': case '5': case '6':
		case '7': case '8': case '9': case '0':
			return true;
		default: return false; }}


bool Parser::nextCharIsLetter() {
	if (read+1 >= (signed) s.size()) return false;

	switch (s[read+1]) {
		case 'a': case 'A': case 'b': case 'B': case 'c': case 'C':
		case 'd': case 'D': case 'e': case 'E': case 'f': case 'F':
		case 'g': case 'G': case 'h': case 'H': case 'i': case 'I':
		case 'j': case 'J': case 'k': case 'K': case 'l': case 'L':
		case 'm': case 'M': case 'N': case 'n': case 'o': case 'O':
		case 'p': case 'P': case 'q': case 'Q': case 'r': case 'R':
		case 's': case 'S': case 't': case 'T': case 'u': case 'U':
		case 'v': case 'V': case 'w': case 'W': case 'x': case 'X':
		case 'y': case 'Y': case 'z': case 'Z':
			return true;
		default: return false; }}


char Parser::nextChar() {
	read++;
	if (read >= (signed) s.size()) return '\0';
	return s[read]; }


std::string* Parser::clearBadChars(std::string &s) {
	std::string *result = new std::string(s);

	for (int i = 0; i < (signed) result->size(); i++) {
		switch ((*result)[i]) {
			case ' ': case '\t': case '\n': case '(': case ')':
			case 'a': case 'A': case 'b': case 'B': case 'c': case 'C':
			case 'd': case 'D': case 'e': case 'E': case 'f': case 'F':
			case 'g': case 'G': case 'h': case 'H': case 'i': case 'I':
			case 'j': case 'J': case 'k': case 'K': case 'l': case 'L':
			case 'm': case 'M': case 'N': case 'n': case 'o': case 'O':
			case 'p': case 'P': case 'q': case 'Q': case 'r': case 'R':
			case 's': case 'S': case 't': case 'T': case 'u': case 'U':
			case 'v': case 'V': case 'w': case 'W': case 'x': case 'X':
			case 'y': case 'Y': case 'z': case 'Z': case '-': case '>':
			case ',': case '|': case '1': case '2': case '3': case '4':
			case '5': case '6': case '7': case '8': case '9': case '0':
				break;
			default:
				(*result)[i] = ' '; }}

	return result; }

	
enum token Parser::getNextToken() {
	readBlank();
	lastTokenInit = read;

	if (nextTokenIsOpen()) return TOKEN_OPEN;
	if (nextTokenIsClose()) return TOKEN_CLOSE;
	if (nextTokenIsImply()) return TOKEN_IMPLY;
	if (nextTokenIsAnd()) return TOKEN_AND;
	if (nextTokenIsOr()) return TOKEN_OR;
	if (nextTokenIsBox()) return TOKEN_BOX;
	if (nextTokenIsNot()) return TOKEN_NOT;
	if (nextTokenIsComma()) return TOKEN_COMMA;
	if (nextTokenIsDash()) return TOKEN_DASH;
	if (nextTokenIsTerminal()) return TOKEN_TERMINAL;

	if (nextCharIsLetter()) return TOKEN_IDENTIF;
	
	return TOKEN_ERROR; }


void Parser::rollBackLastToken() {
	read = lastTokenInit; }


bool Parser::nextTokenIsComma() {
	if (read+1 >= (signed) s.size()) return false;
	if (s[read+1] != ',') return false;

	nextChar();

	return true; }


bool Parser::nextTokenIsDash() {
	if (read+2 >= (signed) s.size()) return false;
	if (s[read+1] != '|' || s[read+2] != '-') return false;

	nextChar();
	nextChar();

	return true; }


bool Parser::nextTokenIsImply() {
	if (read+3 >= (signed) s.size()) return false;
	if (s[read+1] != '-' || s[read+2] != '>') return false;
	if (!isBlank(s[read+3]) && s[read+3] != '(') return false;
	
	nextChar();
	nextChar();
	
	return true; }


bool Parser::nextTokenIsOpen() {
	if (read+1 >= (signed) s.size()) return false;
	if (s[read+1] != '(') return false;
	
	nextChar();
	return true; }


bool Parser::nextTokenIsClose() {
	if (read+1 >= (signed) s.size()) return false;
	if (s[read+1] != ')') return false;
	
	nextChar();
	return true; }


bool Parser::nextTokenIsAnd() {
	if (read+4 >= (signed) s.size()) return false;
	if (s[read+1] != 'A' && s[read+1] != 'a') return false;
	if (s[read+2] != 'n' || s[read+3] != 'd') return false;
	if (!isBlank(s[read+4]) && s[read+4] != '(') return false;
	
	nextChar();
	nextChar();
	nextChar();
	
	return true; }


bool Parser::nextTokenIsOr() {
	if (read+3 >= (signed) s.size()) return false;
	if (s[read+1] != 'O' && s[read+1] != 'o') return false;
	if (s[read+2] != 'r') return false;
	if (!isBlank(s[read+3]) && s[read+3] != '(') return false;
	
	nextChar();
	nextChar();
	
	return true; }


bool Parser::nextTokenIsNot() {
	if (read+4 >= (signed) s.size()) return false;
	if (s[read+1] != 'N' && s[read+1] != 'n') return false;
	if (s[read+2] != 'o' || s[read+3] != 't') return false;
	if (!isBlank(s[read+4]) && s[read+4] != '(') return false;

	nextChar();
	nextChar();
	nextChar();

	return true; }


bool Parser::nextTokenIsBox() {
	if (read+4 >= (signed) s.size()) return false;
	if (s[read+1] != 'B' && s[read+1] != 'b') return false;
	if (s[read+2] != 'o' || s[read+3] != 'x') return false;
	if (!isBlank(s[read+4]) && s[read+4] != '(') return false;

	nextChar();
	nextChar();
	nextChar();

	return true; }


bool Parser::nextTokenIsTerminal() {
	if (read+1 >= (signed) s.size()) return false;
	if (s[read+1] != '#' && s[read+1] != 'n') return false;

	nextChar();

	return true; }


std::string Parser::tokenString(TokenType t) {
	switch (t) {
		case TOKEN_OPEN:
			return "TOKEN_OPEN";
		case TOKEN_NOT:
			return "TOKEN_NOT";
		case TOKEN_BOX:
			return "TOKEN_BOX";
		case TOKEN_AND:
			return "TOKEN_AND";
		case TOKEN_OR:
			return "TOKEN_OR";
		case TOKEN_IDENTIF:
			return "TOKEN_IDENTIF";
		case TOKEN_COMMA:
			return "TOKEN_COMMA";
		case TOKEN_DASH:
			return "TOKEN_DASH";
		case TOKEN_CLOSE:
			return "TOKEN_CLOSE";
		case TOKEN_TERMINAL:
			return "TOKEN_TERMINAL";
		case TOKEN_ERROR:
			return "TOKEN_ERROR";
		default:
			return "UNKNOWN TOKEN"; }}


Formula* Parser::createFormulaTrue() {
	return new FormulaTrue(); }


Formula* Parser::createFormulaFalse() {
	return new FormulaFalse(); }


Formula* Parser::createFormulaIdentif(int IDValue) {
	return new FormulaIdentif(IDValue); }


Formula* Parser::createFormulaBox(Formula *element) {
	return new FormulaBox(element); }


Formula* Parser::createFormulaAnd(std::vector<Formula*> *elements) {
	return new FormulaAnd(elements); }


Formula* Parser::createFormulaXor(std::vector<Formula*> *elements) {
	return new FormulaXor(elements); }


Formula* Parser::createSequent(std::vector<Formula*> *premises, Formula *conclusion) {
	return new Sequent(premises, conclusion); }

