# include "../main/utilities.h"

# include "../formula/formula.h"
# include "parser.h"
# include "parserTest.h"



static std::string name = "parserTest::";


ParserTest::ParserTest() : TestSuite() {
    addTest(new TestParserIdentif());
    addTest(new TestParserNot());
    addTest(new TestParserAnd());
    addTest(new TestParserAndBig());
    addTest(new TestParserOr());
    addTest(new TestParserOrBig());
    addTest(new TestParserImply());
    addTest(new TestParserSequent());
    addTest(new TestParserSequentBig ()); }



ParserTest::~ParserTest() { }


ParserTest::TestParserIdentif::TestParserIdentif() : Test() { }


ParserTest::TestParserIdentif::~TestParserIdentif() { }


bool ParserTest::TestParserIdentif::runTestAfterFork () {
	return testParserIdentif(); }


bool ParserTest::TestParserIdentif::testParserIdentif() {
	std::string function = "testParserIdentif:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "x0123";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaIdentif *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserNot::TestParserNot() : Test() { }


ParserTest::TestParserNot::~TestParserNot() { }


bool ParserTest::TestParserNot::runTestAfterFork () {
	return testParserNot(); }


bool ParserTest::TestParserNot::testParserNot() {
	std::string function = "testParserNot:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "Not(x)";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaXor *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserAnd::TestParserAnd() : Test() { }


ParserTest::TestParserAnd::~TestParserAnd() { }


bool ParserTest::TestParserAnd::runTestAfterFork () {
	return testParserAnd(); }


bool ParserTest::TestParserAnd::testParserAnd() {
	std::string function = "testParserAnd:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "(x)And(z)";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaAnd *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserAndBig::TestParserAndBig() : Test() { }


ParserTest::TestParserAndBig::~TestParserAndBig() { }


bool ParserTest::TestParserAndBig::runTestAfterFork () {
	return testParserAndBig(); }


bool ParserTest::TestParserAndBig::testParserAndBig() {
	std::string function = "testParserAndBig:";
	int testNumber = 0;
	bool failed = failed;
	std::string s = "(w)And(x)And(y)And(z)";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaAnd *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

    testNumber++;
	if (!failed && f->getNumberOfElements() != 4) {
		std::cout << name + function << testNumber << " wrong number of "
			"elements found\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserOr::TestParserOr() : Test() { }


ParserTest::TestParserOr::~TestParserOr() { }


bool ParserTest::TestParserOr::runTestAfterFork () {
	return testParserOr(); }


bool ParserTest::TestParserOr::testParserOr() {
	std::string function = "testParserOr:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "(a)Or(b)";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaXor *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

    testNumber++;
	if (!failed && f->getNumberOfElements() != 2) {
		std::cout << name + function << testNumber << " wrong number of "
			"elements found\n";
		failed = true; }

    testNumber++;
	if(!failed) {
		Formula *fl = f->getElement(1);
		if (fl == NULL || dynamic_cast<FormulaTrue *>(fl) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"second element\n";
			failed = true; }}

    testNumber++;
	if(!failed) {
		Formula *fl = f->getElement(0);
		if(fl == NULL || dynamic_cast<FormulaAnd *>(fl) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"first element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fl = f->getElement(0);
		if (fl->getNumberOfElements() != 2) {
			std::cout << name + function << testNumber << " wrong number of "
				"elements found in first element\n";
			failed = true; }}

    testNumber++;
	if(!failed) {
		Formula *fll = f->getElement(0)->getElement(0);
		if(fll == NULL || dynamic_cast<FormulaXor *>(fll) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"first element of first element\n";
			failed = true; }}

    testNumber++;
	if(!failed) {
		Formula *fll = f->getElement(0)->getElement(1);
		if(fll == NULL || dynamic_cast<FormulaXor *>(fll) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"second element of first element\n";
			failed = true; }}

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserOrBig::TestParserOrBig() : Test() { }


ParserTest::TestParserOrBig::~TestParserOrBig() { }


bool ParserTest::TestParserOrBig::runTestAfterFork () {
	return testParserOrBig(); }


bool ParserTest::TestParserOrBig::testParserOrBig() {
	std::string function = "testParserOrBig:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "(a)Or(b)Or(c)Or(d)";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaXor *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

    testNumber++;
	if (!failed && f->getNumberOfElements() != 2) {
		std::cout << name + function << testNumber << " wrong number of "
			"elements found\n";
		failed = true; }

    testNumber++;
	if (!failed) {
		Formula *fl = f->getElement(1);
		if(fl == NULL || dynamic_cast<FormulaTrue *>(fl) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"second element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fl = f->getElement(0);
		if(fl == NULL || dynamic_cast<FormulaAnd *>(fl) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"first element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fl = f->getElement(0);
		if (fl->getNumberOfElements() != 4) {
			std::cout << name + function << testNumber << " wrong number of "
				"elements found in first element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fll = f->getElement(0)->getElement(0);
		if(fll == NULL || dynamic_cast<FormulaXor *>(fll) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"first element of first element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fll = f->getElement(0)->getElement(1);
		if(fll == NULL || dynamic_cast<FormulaXor *>(fll) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"second element of first element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fll = f->getElement(0)->getElement(2);
		if(fll == NULL || dynamic_cast<FormulaXor *>(fll) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"third element of first element\n";
			failed = true; }}

    testNumber++;
	if (!failed) {
		Formula *fll = f->getElement(0)->getElement(3);
		if(fll == NULL || dynamic_cast<FormulaXor *>(fll) == NULL) {
			std::cout << name + function << testNumber << " wrong formula type of"
				"fourth element of first element\n";
			failed = true; }}

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserImply::TestParserImply() : Test() { }


ParserTest::TestParserImply::~TestParserImply() { }


bool ParserTest::TestParserImply::runTestAfterFork () {
	return testParserImply(); }


bool ParserTest::TestParserImply::testParserImply() {
	std::string function = "testParserImply:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "(x)->(z)";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<FormulaXor *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserSequent::TestParserSequent() : Test() { }


ParserTest::TestParserSequent::~TestParserSequent() { }


bool ParserTest::TestParserSequent::runTestAfterFork () {
	return testParserSequent(); }


bool ParserTest::TestParserSequent::testParserSequent() {
	std::string function = "testParserSequent:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "x|-x";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<Sequent *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }


ParserTest::TestParserSequentBig::TestParserSequentBig() : Test() { }


ParserTest::TestParserSequentBig::~TestParserSequentBig() { }


bool ParserTest::TestParserSequentBig::runTestAfterFork () {
	return testParserSequentBig(); }


bool ParserTest::TestParserSequentBig::testParserSequentBig() {
	std::string function = "testParserSequentBig:";
	int testNumber = 0;
	bool failed = false;
	std::string s = "a,(a)->(b),(b)->(c),(c)->(d)|- d";
	Parser *p = new Parser(s);
	Formula *f;
	f = p->parse();

    testNumber++;
	if (!failed && f == NULL) {
		std::cout << name + function << testNumber << " NULL returned\n";
		failed = true; }

    testNumber++;
	if (!failed && dynamic_cast<Sequent *>(f) == NULL) {
		std::cout << name + function << testNumber << " wrong formula type\n";
		failed = true; }

	f->erase();
	delete p;
	return !failed; }
